@@ -106,6 +106,11 @@ class GLiNER(nn.Module, PyTorchModelHubMixin):
 
     @property
     def device(self):
+        if self.onnx_model:
+            providers = self.model.session.get_providers()
+            if 'CUDAExecutionProvider' in providers:
+                return torch.device('cuda')
+            return torch.device('cpu')
         device = next(self.model.parameters()).device
         return device
 
@@ -209,13 +214,12 @@ class GLiNER(nn.Module, PyTorchModelHubMixin):
             }
         )
 
-        if not self.onnx_model:
-            device = self.device
-            for key in model_input:
-                if model_input[key] is not None and isinstance(
-                    model_input[key], torch.Tensor
-                ):
-                    model_input[key] = model_input[key].to(device)
+        device = self.device
+        for key in model_input:
+            if model_input[key] is not None and isinstance(
+                model_input[key], torch.Tensor
+            ):
+                model_input[key] = model_input[key].to(device)
 
         return model_input, raw_batch
 
@@ -321,7 +325,7 @@ class GLiNER(nn.Module, PyTorchModelHubMixin):
         # raw input preparation
         input_x, all_start_token_idx_to_text_idx, all_end_token_idx_to_text_idx = self.prepare_texts(texts)
 
-        labels = list(dict.fromkeys(labels))
+        # labels = list(dict.fromkeys(labels))
 
         collator = DataCollator(
             self.config,
@@ -340,9 +344,10 @@ class GLiNER(nn.Module, PyTorchModelHubMixin):
         # Iterate over data batches
         for batch in data_loader:
             # Move the batch to the appropriate device
-            for key in batch:
-                if isinstance(batch[key], torch.Tensor):
-                    batch[key] = batch[key].to(self.device)
+            if not self.onnx_model:
+                for key in batch:
+                    if isinstance(batch[key], torch.Tensor):
+                        batch[key] = batch[key].to(self.device)
 
             # Perform predictions
             model_output = self.model(**batch)[0]
@@ -507,7 +512,6 @@ class GLiNER(nn.Module, PyTorchModelHubMixin):
             dataset, batch_size=batch_size, shuffle=False, collate_fn=collator
         )
 
-        device = self.device
         all_preds = []
         all_trues = []
 
@@ -516,7 +520,7 @@ class GLiNER(nn.Module, PyTorchModelHubMixin):
             # Move the batch to the appropriate device
             for key in batch:
                 if isinstance(batch[key], torch.Tensor):
-                    batch[key] = batch[key].to(device)
+                    batch[key] = batch[key].to(self.device)
 
             # Perform predictions
             model_output = self.model(**batch)[0]
@@ -652,6 +656,7 @@ class GLiNER(nn.Module, PyTorchModelHubMixin):
         config: Optional[GLiNERConfig] = None,
         repo_id: Optional[str] = None,
         push_to_hub: bool = False,
+        safe_serialization = False,
         **push_to_hub_kwargs,
     ) -> Optional[str]:
         """
@@ -677,6 +682,7 @@ class GLiNER(nn.Module, PyTorchModelHubMixin):
 
         # save model weights/files
         model_state_dict = self.prepare_state_dict(self.model.state_dict())
+        # save model weights using safetensors
         if safe_serialization:
             save_file(model_state_dict, os.path.join(save_directory, "model.safetensors"))
         else:
@@ -778,7 +784,8 @@ class GLiNER(nn.Module, PyTorchModelHubMixin):
                 tokenizer = AutoTokenizer.from_pretrained(model_dir)
             else:
                 tokenizer = None
-        config_ = json.load(open(config_file))
+        with open(config_file, "r") as f:
+            config_ = json.load(f)
         config = GLiNERConfig(**config_)
 
         if _attn_implementation is not None:
@@ -828,7 +835,12 @@ class GLiNER(nn.Module, PyTorchModelHubMixin):
                 session_options.graph_optimization_level = (
                     ort.GraphOptimizationLevel.ORT_ENABLE_ALL
                 )
-            ort_session = ort.InferenceSession(model_file, session_options)
+            providers = ['CPUExecutionProvider']
+            if "cuda" in map_location:
+                if not torch.cuda.is_available():
+                    raise RuntimeError("CUDA is not available but `map_location` is set to 'cuda'.")
+                providers = ['CUDAExecutionProvider']
+            ort_session = ort.InferenceSession(model_file, session_options, providers=providers)
             if config.span_mode == "token_level":
                 model = TokenORTModel(ort_session)
             else:
